@@ -1,5 +1,5 @@
       subroutine glkern_s(t,x, tt,y, n,m,nue,kord, hetero,isrand,
-     .     smo,m1,tl,tu,s,sig,wn,w1,b)
+     .     inputb,m1,tl,tu,s,sig,wn,w1,b, trace)
 c----------------------------------------------------------------------*
 c-----------------------------------------------------------------------
 c       Short-version: Oct 1996
@@ -10,16 +10,16 @@ c       General subroutine for kernel smoothing:
 c       Computation of iterative plug-in algorithm for global bandwidth
 c       selection for kernels with
 c       (nue,kord) = (0,2), (0,4), (1,3) or (2,4).
-c
 c-----------------------------------------------------------------------
 c  used subroutines: constV, resest, kernel with further subroutines
 c-----------------------------------------------------------------------
 c Args
       integer n, m, nue,kord
       double precision t(n),x(n), tt(m), tl,tu, s(0:n), sig
-      logical hetero, isrand, smo
-c 'smo' == 'inputb' : if TRUE, *use* bandwidth 'b' given as input
-      integer m1
+      logical hetero, isrand, inputb
+c			      inputb (was "smo", now same as in R):
+c	 if TRUE, do not compute bandwidth, but *use* 'b' given as input
+      integer m1, trace
       double precision wn(0:n,5),w1(m1,3), b, y(m)
 c Var
       logical inputs, needsrt
@@ -37,22 +37,22 @@ c-------- 1. initialisations
 
 c Stop for invalid inputs (impossible when called from R's lokerns())
 
-c     0 <= nue <= 4;  nue <= 2 if(! smo)
-      if(nue.gt.4.or.nue.lt.0) call rexit("nue must be in 0..4")
-      if(nue.gt.2.and. .not.smo)
-     +     call rexit("nue must be in 0..2 if not 'smo'")
-      if(n.le.2) call rexit("n <= 2")
-      if(m.lt.1) call rexit("m < 1")
-      if(m1.lt.3) call rexit("m1 < 3")
+c     0 <= nue <= 4;  nue <= 2 if(! inputb)
+      if(nue.gt.4 .or. nue.lt.0) call rexit("nue must be in 0..4")
+      if(nue.gt.2 .and. .not. inputb)
+     +     call rexit("nue must be in 0..2 if not 'inputb'")
+      if(n .le. 2) call rexit("n <= 2")
+      if(m .lt. 1) call rexit("m < 1")
+      if(m1.lt. 3) call rexit("m1 < 3")
 
 c     kord - nue must be even  <<--- MM: *UN*desirable (want to fix kord, vary 'nue')!
 c                        ----  <<---     but a short glimpse at coff*() ./auxkerns.f
 c                                        reveals how much work would be needed to change this
       kk=(kord-nue)/2
       if(2*kk + nue .ne. kord)       kord=nue+2
-      if(kord.gt.4 .and. .not.smo)   kord=nue+2
+      if(kord.gt.4 .and. .not.inputb)   kord=nue+2
       if(kord.gt.6 .or. kord.le.nue) kord=nue+2
-      if(smo.and.b.le.0) smo=.false.
+      if(inputb.and.b.le.0) inputb=.false.
       rvar=sig
 c- -Wall (erronously warning if not)
       bmin=1
@@ -62,26 +62,32 @@ c- -Wall (erronously warning if not)
 
       il=1
       iu=n
+
+      if(trace .gt. 0) call monit0(0, n, m, nue, kord, trace)
+
 c-------- 2. computation of s-sequence
+      if(trace .gt. 0) call monit1(2, trace)
       s0=1.5*t(1)-0.5*t(2)
       sn=1.5*t(n)-0.5*t(n-1)
       if(s(n).le.s(0)) then
          inputs= .true.
-         do 20 i=1,n-1
+         do i=1,n-1
             s(i)=.5*(t(i)+t(i+1))
- 20      continue
+         end do
          s(0)=s0
          s(n)=sn
-         if(smo.and. .not.isrand) goto 160
+         if(inputb.and. .not.isrand) goto 160
       else
-         if(smo) goto 160
+         if(inputb) goto 160
       end if
 c-
 c-------- 3. computation of minimal, maximal allowed global bandwidth
+      if(trace .gt. 0) call monit1(3, trace)
       bmax=(sn-s0)*.5
       bmin=(sn-s0)/dble(n)*dble(kord-1)*.6
 c-
 c-------- 4. compute tl,tu
+      if(trace .gt. 0) call monit1(4, trace)
       itt=0
 40    if (tu.le.tl) then
         tl=.933*s0+.067*sn
@@ -92,14 +98,15 @@ c-------- 4. compute tl,tu
       tu=min(sn,tu)
 c-
 c-------- 5. compute indices
+      if(trace .gt. 0) call monit1(5, trace)
       wn(1,1)=0.0
       wn(n,1)=0.0
-      do 50 i=1,n
+      do i=1,n
         if(t(i).le.tl .or.  t(i).ge.tu) wn(i,1)=0.0
         if(t(i).gt.tl .and. t(i).lt.tu) wn(i,1)=1.0
         if(t(i).lt.tl) il=i+1
         if(t(i).le.tu) iu=i
- 50   continue
+      end do
       nn=iu-il+1
       if(nn.eq.0.and.itt.eq.0) then
         tu=tl-1.0
@@ -112,41 +119,45 @@ c-------- 5. compute indices
       end if
 c-
 c-------- 6. compute t-grid for integral approximation
-      do 60 i=1,m1
+      if(trace .gt. 0) call monit1(6, trace)
+      do i=1,m1
          w1(i,2)=1.0
          w1(i,1)=tl+(tu-tl)*dble(i-1)/dble(m1-1)
- 60   continue
+      end do
 c-
 c-------- 7. calculation of weight function
+      if(trace .gt. 0) call monit1(7, trace)
       alpha=1.d0/dble(13)
-      do 70 i=il,iu
+      do i=il,iu
          xi=(t(i) - tl)/alpha/(tu-tl)
          if(xi.gt.1) goto 71
          wn(i,1)=(10.0-15*xi+6*xi*xi)*xi*xi*xi
- 70   continue
- 71   do 72 i=iu,il,-1
+      end do
+ 71   do i=iu,il,-1
         xi=(tu-t(i))/alpha/(tu-tl)
         if(xi.gt.1) goto 73
         wn(i,1)=(10.0-15*xi+6*xi*xi)*xi*xi*xi
- 72   continue
- 73   do 74 i=1,m1
+      end do
+ 73   do i=1,m1
          xi=(w1(i,1)-tl)/alpha/(tu-tl)
          if(xi.gt.1) goto 75
          w1(i,2)=(10.0-15*xi+6*xi*xi)*xi*xi*xi
- 74   continue
- 75   do 76 i=m1,1,-1
+      end do
+ 75   do i=m1,1,-1
          xi=(tu-w1(i,1))/alpha/(tu-tl)
          if(xi.gt.1) goto 77
          w1(i,2)=(10.0-15*xi+6*xi*xi)*xi*xi*xi
- 76   continue
+      end do
  77   continue
 c-
 c-------- 8. compute constants for iteration
+      if(trace .gt. 0) call monit1(8, trace)
       ex=1./dble(kord+kord+1)
       kk2=(kord-nue)
       kk=kk2/2
 c-
 c-------- 9. estimating variance and smoothed pseudoresiduals
+      if(trace .gt. 0) call monit1(9, trace)
       rvar=sig
       if(sig .le. 0. .and. .not.hetero) then
         call resest(t(il),x(il),nn,wn(il,2),r2,sig)
@@ -154,52 +165,55 @@ c-------- 9. estimating variance and smoothed pseudoresiduals
       if(hetero) then
          call resest(t,x,n,wn(1,2),snr,sig)
          bres=max(bmin,.2*nn**(-.2)*(s(iu)-s(il-1)))
-         do 91 i=1,n
+         do i=1,n
             wn(i,3)=t(i)
             wn(i,2)=wn(i,2)*wn(i,2)
- 91      continue
+         end do
          call kernel(t,wn(1,2),n,bres,0,kk2,nyg,s,
-     .        wn(il,3),nn,wn(il,4))
+     .        wn(il,3),nn,wn(il,4), trace)
       else
 c       not hetero
         call constV(wn(1,4),n,sig)
       end if
 c-
 c-------- 10. [LOOP:] estimate/compute integral constant
+      if(trace .gt. 0) call monit1(10, trace)
 100   vi=0.
-      do 101 i=il,iu
+      do i=il,iu
          vi=vi+ wn(i,1)*n*(s(i)-s(i-1))**2 * wn(i,4)
- 101  continue
+      end do
 c-
 c-------- 11. refinement of s-sequence for random design
+      if(trace .ge. 2) call monit1(11, trace)
       if(inputs .and. isrand) then
-        do 110 i=0,n
+        do i=0,n
           wn(i,5)=dble(i)/dble(n+1)
           wn(i,2)=(dble(i)+.5)/dble(n+1)
           wn(i,3)=wn(i,2)
- 110    continue
+        end do
         exs= -dble(3*kord+1) / dble(6*kord+3)
         exsvi=dble(kord)     / dble(6*kord+3)
         bs=0.1*(vi/(sn-s0)**2)**exsvi * n**exs
-        call kernel(wn(1,5),t,n,bs,0,2,nyg,wn(0,3),wn(0,2),n+1,s(0))
-
+        call kernel(wn(1,5),t,n,bs,0,2,nyg,wn(0,3),wn(0,2),n+1,s(0),
+     .       trace)
         vi=0.0
 111     needsrt=.false.
-        do 112 i=1,n
-           vi=vi+wn(i,1)*n*(s(i)-s(i-1))**2*wn(i,4)
+        do i=1,n
+           vi=vi+ wn(i,1)*n*(s(i)-s(i-1))**2 * wn(i,4)
            if(s(i).lt.s(i-1)) then
               ssi=s(i-1)
               s(i-1)=s(i)
               s(i)=ssi
               needsrt=.true.
            end if
-112     continue
+        end do
         if(needsrt) goto 111
-        if(smo) goto 160
+        if(inputb) goto 160
       end if
       b=bmin*2.
 c-
 c-------- 12. compute inflation constant and exponent and loop of iterations
+      if(trace .ge. 2) call monit1(12, trace)
       const=dble(2*nue+1)*fak2(kord)*vark(kk,nue)*vi
      .       /(dble(2*kord-2*nue)*bias(kk,nue)**2*dble(n))
       fac=1.1*(1.+(nue/10.)+0.05*(kord-nue-2.))
@@ -209,32 +223,37 @@ c     itende=1+2*kord+kord*(2*kord+1)
       itende = (1 + 2*kord) * (1 + kord)
 c     ^^^^^^  *fixed* number of iterations ( <== theory !)
 
-      do 120 it=1,itende
+      do it=1,itende
 c-
 c-------- 13. estimate derivative of order kord in iterations
+        if(trace .ge. 3) call monit1(13, trace)
         b2=b*fac
         b2=max(b2,bmin/dble(kord-1)*dble(kord+1))
         b2=min(b2,bmax)
-        call kernel(t,x,n,b2,kord,kord+2,nyg,s,w1(1,1),m1,w1(1,3))
+        call kernel(t,x,n,b2,kord,kord+2,nyg,s,w1(1,1),m1,w1(1,3),trace)
 c-
 c-------- 14. estimate integralfunctional in iterations
-        xmy2=.75*(w1(1,2)*w1(1,3)*w1(1,3)+w1(m1,2)*w1(m1,3)*w1(m1,3))
-        do 140 i=2,m1-1
-140       xmy2=xmy2+w1(i,2)*w1(i,3)*w1(i,3)
+        if(trace .ge. 3) call monit1(14, trace)
+        xmy2= .75*(w1(1,2)*w1(1,3)*w1(1,3) + w1(m1,2)*w1(m1,3)*w1(m1,3))
+        do i=2,m1-1
+           xmy2=xmy2+w1(i,2)*w1(i,3)*w1(i,3)
+        end do
         xmy2=xmy2*(tu-tl)/dble(m1)
 c-
 c-------- 15. finish of iterations
+        if(trace .ge. 3) call monit1(15, trace)
         b=(const/xmy2)**ex
         b=max(bmin,b)
         b=min(bmax,b)
-120   continue
+      end do
 
 c-------- 16  compute smoothed function with global plug-in bandwidth
-160   call kernel(t,x,n,b,nue,kord,nyg,s,tt,m,y)
+ 160  if(trace .ge. 2) call monit1(16, trace)
+      call kernel(t,x,n,b,nue,kord,nyg,s,tt,m,y, trace)
 c-  return #{iter} (iff aplicable)
       m1=itende
 c-------- 17. variance check
-      irnd=1-irnd
+      if(trace .ge. 2) call monit1(17, trace)
       if(hetero) sig=rvar
       if(hetero.or. r2.lt.0.88 .or. nue.gt.0) return
       ii=0
